/**
 **   xpcsong.c - xPC Target device driver to generate frequencies
 **
 **   Copyright 2008 The MathWorks, Inc.
 **/


#define S_FUNCTION_LEVEL   2
#undef  S_FUNCTION_NAME
#define S_FUNCTION_NAME    xpcsong

#include <stddef.h>
#include <stdlib.h>
#include "simstruc.h"

#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif

enum {
    S_FREQUENCIES = 0,
    S_DURATIONS,
    S_INTERMISSION,
    S_SAMPLE_TIME,
    NUM_S_PARAMS
};

enum {
    W_SAMPLES = 0,
    W_SILENCE,
    W_FREQUENCY,
    NUM_WORK_VARS
};


static void mdlInitializeSizes(SimStruct *S)
{
   int_T j;

   ssSetNumSFcnParams(S, NUM_S_PARAMS);
   if ( ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S) )
        return;

   ssSetNumContStates(S, 0);
   ssSetNumDiscStates(S, 0);

   ssSetNumInputPorts(S, 0);

   ssSetNumOutputPorts(S, 1);
   ssSetOutputPortDataType(S, 0, SS_DOUBLE);
   ssSetOutputPortWidth(S, 0, 1);

   ssSetNumIWork(S, NUM_WORK_VARS);
   ssSetNumPWork(S, 0);
   ssSetNumRWork(S, 0);
   ssSetNumModes(S, 0);

   ssSetNumSampleTimes(S, 1);
   ssSetNumNonsampledZCs(S, 0);

   for ( j = 0 ; j < NUM_S_PARAMS ; j++ )
      ssSetSFcnParamTunable(S, j, SS_PRM_NOT_TUNABLE);

   ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    time_T sample_time = (time_T)mxGetPr(ssGetSFcnParam(S, S_SAMPLE_TIME))[0];

    if ( sample_time == -1.0 ) {
        ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
        ssSetOffsetTime(S, 0, FIXED_IN_MINOR_STEP_OFFSET);
    } else {
        ssSetSampleTime(S, 0, sample_time);
        ssSetOffsetTime(S, 0, 0.0);
    }
}

#define MDL_START
static void mdlStart(SimStruct *S)
{
   ssSetIWorkValue(S, W_SAMPLES, -1);
   ssSetIWorkValue(S, W_SILENCE, -1);
   ssSetIWorkValue(S, W_FREQUENCY, -1);

   if ( ((int_T)mxGetN(ssGetSFcnParam(S, S_FREQUENCIES)) <= 0) ||
        ((int_T)mxGetN(ssGetSFcnParam(S, S_DURATIONS)) <= 0) )
      ssSetIWorkValue(S, W_SILENCE, 1);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
   real_T frequency;
   real_T duration;
   real_T intermission;
   int32_T samples          = (int32_T)ssGetIWorkValue(S, W_SAMPLES);
   int32_T silence          = (int32_T)ssGetIWorkValue(S, W_SILENCE);
   int32_T currentFrequency = (int32_T)ssGetIWorkValue(S, W_FREQUENCY);
   uint32_T frequencySize = (uint32_T)mxGetN(ssGetSFcnParam(S, S_FREQUENCIES));
   uint32_T durationSize  = (uint32_T)mxGetN(ssGetSFcnParam(S, S_DURATIONS));
   real_T sampleTime = (real_T)ssGetSampleTime(S, 0);
   real_T *frequencyOut = ssGetOutputPortRealSignal(S, 0);

   if ( silence > 0 ) {
      *frequencyOut = 0.0;
      silence--;
      if ( (frequencySize <= 0) || (durationSize <= 0) )
         silence = 1;
      ssSetIWorkValue(S, W_SILENCE, silence);
      return;
   }

   if ( samples <= 0 ) {
      currentFrequency++;
      duration = (real_T)mxGetPr(ssGetSFcnParam(S, S_DURATIONS))[currentFrequency];
      frequency = (real_T)mxGetPr(ssGetSFcnParam(S, S_FREQUENCIES))[currentFrequency];
      samples = duration / sampleTime;
      *frequencyOut = 0.0 - frequency;
   } else {
      frequency = (real_T)mxGetPr(ssGetSFcnParam(S, S_FREQUENCIES))[currentFrequency];
      *frequencyOut = frequency;
   }

   samples--;

   if ( (samples <= 0) &&
        (((currentFrequency + 1) >= frequencySize) ||
         ((currentFrequency + 1) >= durationSize)) ) {
      intermission = (real_T)mxGetPr(ssGetSFcnParam(S, S_INTERMISSION))[0];
      silence = intermission / sampleTime;
      samples  = -1;
      currentFrequency = -1;
   }

   ssSetIWorkValue(S, W_SAMPLES, samples);
   ssSetIWorkValue(S, W_SILENCE, silence);
   ssSetIWorkValue(S, W_FREQUENCY, currentFrequency);
}

static void mdlTerminate(SimStruct *S)
{
}


#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif // MATLAB_MEX_FILE
